Input = {
    --move input
    UP = 0x00000001,
    DOWN = 0x00000002,
    BACK = 0x00000004,
    FORWARD = 0x00000008,
    --combo inpt
    L_HAND = 0x00004000,
    R_HAND = 0x00008000,
    L_KICK = 0x00001000,
    R_KICK = 0x00002000,
    L_B = 0x00000200
}

InputType = {
    KEY_DOWN = 0,
    KEY_UP = 1,
    KEY_PRESS = 2
}

Keys = {
    A = 65, -- A key
    B = 66, -- B key
    C = 67, -- C key
    D = 68, -- D key
    E = 69, -- E key
    F = 70, -- F key
    G = 71, -- G key
    H = 72, -- H key
    I = 73, -- I key
    J = 74, -- J key
    K = 75, -- K key
    L = 76, -- L key
    M = 77, -- M key
    N = 78, -- N key
    O = 79, -- O key
    P = 80, -- P key
    Q = 81, -- Q key
    R = 82, -- R key
    S = 83, -- S key
    T = 84, -- T key
    U = 85, -- U key
    V = 86, -- V key
    W = 87, -- W key
    X = 88, -- X key
    Y = 89, -- Y key
    Z = 90 -- Z key
}

Vector3 = {}

InputDirectionCodes = {
    NA = 0,
    N = 0x20,
    u = 0x100,
    ub = 0x80,
    uf = 0x200,
    f = 0x40,
    b = 0x10,
    d = 4,
    df = 8,
    db = 2
}

InputAttackCodes = {
    N = 0,
    x1 = 512,
    x2 = 1024,
    x3 = 2048,
    x4 = 4096,
    x1x2 = 1536,
    x1x3 = 2560,
    x1x4 = 4608,
    x2x3 = 3072,
    x2x4 = 5120,
    x3x4 = 6144,
    x1x2x3 = 3584,
    x1x2x4 = 5632,
    x1x3x4 = 6656,
    x2x3x4 = 7168,
    x1x2x3x4 = 7680,
    xRAGE = 8192
}

AttackType = {
    ANTIAIR_ONLY = 11, -- Attack type that only affects airborne opponents, e.g., Alisa's chainsaw stance f+2
    THROW = 10, -- Attack type that only works during a throw animation
    LOW_UNBLOCKABLE = 9, -- Attack type that cannot be blocked low, e.g., Yoshimitsu's 10-hit 2
    HIGH_UNBLOCKABLE = 8, -- Attack type that cannot be blocked high, e.g., Akuma's charged attack
    MID_UNBLOCKABLE = 7, -- Attack type that cannot be blocked mid
    UNKNOWN = 6, -- Unknown attack type (may not exist)
    HIGH = 5, -- High attack type
    SMID = 4, -- Special mid attack type
    PROJ = 3, -- Projectile attack type, special mid attack that cannot be parried. Unknown if it has other common properties.
    MID = 2, -- Mid attack type
    LOW = 1, -- Low attack type
    NA = 0 -- Non-attack type, the action is not an attack
}

SimpleMove = {
    UNINITIALIZED = 0, -- Move state uninitialized
    STANDING_FORWARD = 1, -- Moving forward while standing
    STANDING_BACK = 2, -- Moving backward while standing
    STANDING = 3, -- Standing state
    STEVE = 4, -- Steve?
    CROUCH_FORWARD = 5, -- Moving forward while crouching
    CROUCH_BACK = 6, -- Moving backward while crouching
    CROUCH = 7, -- Crouching state
    UNKNOWN_TYPE_9 = 9, -- Seen on Ling
    GROUND_FACEUP = 12, -- Grounded face-up state
    GROUND_FACEDOWN = 13, -- Grounded face-down state
    JUGGLED = 14, -- Juggled state
    KNOCKDOWN = 15, -- Knockdown state
    OFF_AXIS_GETUP = 8, -- Understanding of the following values is incomplete
    UNKNOWN_10 = 10, -- Yoshimitsu
    UNKNOWN_GETUP_11 = 11,
    WALL_SPLAT_18 = 18,
    WALL_SPLAT_19 = 19,
    TECH_ROLL_OR_FLOOR_BREAK = 20,
    UNKNOWN_23 = 23, -- Kuma
    AIRBORNE_24 = 24, -- Yoshimitsu
    AIRBORNE = 25,
    AIRBORNE_26 = 26, -- Eliza, Chloe
    FLY = 27 -- Devil Jin 3+4
}

ComplexMove = {
    F_MINUS = 0, -- Represents both empty state and attack startup. # Happens in Kazuya's hell sweep
    S_PLUS = 1, -- Tracking
    S = 2, -- Tracking, usually with a screw, seems slightly easier to deviate from the axis?
    A = 3, -- This action 'realigns' if you pause before the throw
    UN04 = 4, -- Extremely rare, Eliza's front front kick has this
    C_MINUS = 5, -- Realigning, either worse than C or slightly better, hard to say
    A_PLUS = 6, -- Realignment is very good # Alisa's b+2 1 has this, very rare
    C = 7, -- This realignment is worse than 'A'
    END1 = 10, -- After startup ### Kazuya's front front jump has no startup or attack end mark, the whole process is 0 ??? ### Lily's d/b+4 is blocked and no
    BLOCK = 11, -- In defense
    WALK = 12, -- Used for dashing and walking
    SIDEROLL_GETUP = 13, -- Only happens after rolling sideways???
    SIDEROLL_STAYDOWN = 14,
    SS = 15, -- Side step left or right, also used for striking techniques
    RECOVERING = 16, -- Occurs when stopping forward or backward movement, jumping, being hit, entering a stance, etc.
    UN17 = 17, -- f+4 with Ling
    UN18 = 18, -- King's 1+2+3+4 luck charge
    UN20 = 20, -- Dragunov's d+3+4 ground jump
    UN22 = 22, -- Eddy's move
    UN23 = 23, -- Steve 3+4 1
    SW = 28, -- Sidewalk left or right
    BROKEN_54 = 54,
    UNKN = 999999 -- Used to indicate non-standard tracking actions
}

StunType = {
    NONE = 0, -- No stun status
    UNKNOWN_2 = 2, -- Unknown status 2 (possibly used to describe certain skills, such as Lili's BT (back turned) state, jumping, or kicking)
    BLOCK = 16777472, -- Blocking status
    GETTING_HIT = 256, -- Being hit status
    DOING_THE_HITTING = 65536, -- Attacking status
    BEING_PUNISHED = 65792, -- Being punished status (the starting frame of the punishment state, also occurs when being countered at the same time)
    BLOCK_NO_HIT = 16777216 -- Blocking without being hit status (possibly used to describe certain skills, such as Law's UF+4, sometimes may be close to defense)
}

HitType = {
    NONE = 0, -- Not hit
    BLOCKED_STANDING = 1, -- Blocked while standing
    BLOCKED_CROUCHING = 2, -- Blocked while crouching
    JUGGLE = 3, -- Juggle state
    SCREW = 4, -- Screw state
    UNKNOWN_SCREW_5 = 5, -- Unknown hit state 5 (possibly related to Xiaoyu's demo combo 3 ending, may be off-axis or right side state)
    UNKNOWN_6 = 6, -- Unknown hit state 6 (possibly non-existent)
    UNKNOWN_SCREW_7 = 7, -- Unknown screw hit state 7 (Xiaoyu's demo combo 3 includes this state)
    GROUNDED_FACE_DOWN = 8, -- Grounded, facing down
    GROUNDED_FACE_UP = 9, -- Grounded, facing up
    COUNTER_HIT_STANDING = 10, -- Counter hit while standing
    COUNTER_HIT_CROUCHING = 11, -- Counter hit while crouching
    NORMAL_HIT_STANDING = 12, -- Normal hit while standing
    NORMAL_HIT_CROUCHING = 13, -- Normal hit while crouching
    NORMAL_HIT_STANDING_LEFT = 14, -- Normal hit on the left while standing
    NORMAL_HIT_CROUCHING_LEFT = 15, -- Normal hit on the left while crouching
    NORMAL_HIT_STANDING_BACK = 16, -- Normal hit on the back while standing
    NORMAL_HIT_CROUCHING_BACK = 17, -- Normal hit on the back while crouching
    NORMAL_HIT_STANDING_RIGHT = 18, -- Normal hit on the right while standing
    NORMAL_HIT_CROUCHING_RIGHT = 19 -- Normal hit on the right while crouching
}

ThrowType = {
    NONE = 0, -- No throw
    TE1 = 3489660956, -- Throw escape 1
    TE2 = 3758096415, -- Throw escape 2
    TE1_2 = 4026531870, -- Throw escape 1+2
    TE1_3 = 2415919128,
    BROKEN = 3221225501 -- Broken throw
}

PlayerSnapshot = {
    move_timer = 0, -- 移动计时器
    move_id = 0, -- 移动编号
    simple_state = 0, -- 简单状态
    attack_type = 0, -- 攻击类型
    startup = 0, -- 启动时间
    startup_end = 0, -- 结束时间
    attack_damage = 0, -- 攻击伤害
    complex_state = 0, -- 复杂状态
    damage_taken = 0, -- 承受的伤害
    recovery = 0, -- 恢复时间
    frames_til_next_move = 0, -- 直到下一次移动的帧数
    char_id = 0, -- 角色编号
    distance = 0.0, -- 距离
    jump_flag = 0, -- 跳跃标志
    throw_tech = 0, -- 投掷技术
    input_direction = 0, -- 输入方向
    input_attack = 0, -- 输入攻击
    stun_state = 0, -- 眩晕状态
    hit_outcome = 0, -- 击中结果
    facing = 0 -- 朝向
}

GameSnapshot = {
    self = PlayerSnapshot,
    enemy = PlayerSnapshot,
    frame_count = 0,
    facing_bool = 0
}
