return {
    -- 蹲姿起身瞬间发动攻击，惩罚对手攻击
    IsWSpunish = function(self, enemy)
        -- 检查玩家是否处于向后蹲（CROUCH_BACK）或蹲姿（CROUCH）状态
        if
            (self.simple_state == SimpleMove.CROUCH_BACK or self.simple_state == self.simple_state == SimpleMove.CROUCH) and
                (enemy.attack_type ~= AttackType.MID or enemy.attack_type ~= AttackType.HIGH) and
                self.complex_state == ComplexMove.BLOCK
         then
            return true
        end
        return false
    end,
    -- 特殊状态检查
    IsSpecialStatus = function(self)
        return self.simple_state == SimpleMove.KNOCKDOWN or self.simple_state == SimpleMove.JUGGLED or
            self.complex_state == ComplexMove.SIDEROLL_GETUP or
            self.complex_state == ComplexMove.SIDEROLL_STAYDOWN
    end,
    -- 获取对手攻击到达的时间（帧数）
    GetTimeUntilImpact = function(attacker)
        return attacker.startup - attacker.move_timer + attacker.startup_end - attacker.startup + 1
    end,
    -- 判断是否在墙上
    IsGettingWallSplatted = function(self)
        return self.simple_state == SimpleMove.WALL_SPLAT_18 or self.simple_state == SimpleMove.WALL_SPLAT_19
    end,
    -- 判断是否蹲着
    IsCrouching = function(self)
        return self.simple_state == SimpleMove.CROUCH_FORWARD or self.simple_state == SimpleMove.CROUCH_BACK or
            self.simple_state == SimpleMove.CROUCH
    end,
    -- 判断是否被击倒在地
    IsGettingGroundHit = function(self)
        return self.hit_outcome == HitType.GROUNDED_FACE_DOWN or self.hit_outcome == HitType.GROUNDED_FACE_UP
    end,
    IsCrouchBack = function(self)
        return self.simple_state == SimpleMove.CROUCH_BACK
    end,
    -- 判断是否处于反击状态
    IsGettingCounterHit = function(self)
        return self.hit_outcome == HitType.COUNTER_HIT_CROUCHING or self.hit_outcome == HitType.COUNTER_HIT_STANDING
    end,
    -- 判断攻击是否无法格挡
    IsAttackUnblockable = function(self)
        return self.attack_type == AttackType.HIGH_UNBLOCKABLE or self.attack_type == AttackType.LOW_UNBLOCKABLE or
            self.attack_type == AttackType.MID_UNBLOCKABLE
    end,
    -- 判断是否处于空中
    IsAirborne = function(attacker)
        local airborne_states = {
            SimpleMove.JUGGLED,
            SimpleMove.AIRBORNE_24,
            SimpleMove.AIRBORNE,
            SimpleMove.FLY,
            SimpleMove.AIRBORNE_26
        }
        for _, state in ipairs(airborne_states) do
            if attacker.simple_state == state then
                return true
            end
        end
        return false
    end,
    IsPowerCrush = function(self)
        return self.power_crush == 256
    end,
    -- 判断是否可以进行反击，条件包括：不处于防御状态、对手攻击超过一定帧数、对手处于空中等。
    CanCounter = function(frames, attacker, receiver, airborneOpp)
        -- 不要反击碾压攻击
        if utils.IsPowerCrush(attacker) then
            return false
        end
        -- 如果接收者处于防御状态，无法进行反击 预测模式不使用
        if utils.IsBlocking(receiver) then
            return false
        end
        -- 如果攻击帧数大于10，可以进行反击
        if frames > 10 then
            -- 如果对手在空中且攻击帧数大于5，可以进行反击
            return true
        elseif airborneOpp and frames > 5 then
            -- 其他情况下无法进行反击
            return true
        else
            return false
        end
    end,
    IsAttackStarting = function(self)
        -- 判断是否开始攻击
        if self.startup > 0 then
            local is_active = self.move_timer <= self.startup
            return is_active
        else
            return false
        end
    end,
    IsAttackLow = function(self)
        -- 判断攻击是否为下段
        return self.attack_type == AttackType.LOW
    end,
    IsAttackMid = function(self)
        -- 判断攻击是否为中段
        return self.attack_type == AttackType.MID
    end,
    IsAttackHigh = function(self)
        -- 判断攻击是否为上段
        return self.attack_type == AttackType.HIGH
    end,
    IsAttackThrow = function(self)
        -- 判断攻击是否为投掷
        return self.attack_type == AttackType.THROW
    end,
    IsOnGround = function(self)
        -- 判断是否在地面上
        local ground_states = {
            SimpleMove.GROUND_FACEDOWN,
            SimpleMove.GROUND_FACEUP
        }
        for _, state in ipairs(ground_states) do
            if self.simple_state == state then
                return true
            end
        end
        return false
    end,
    -- 是否正在防御期间
    IsBlocking = function(self)
        return self.complex_state == ComplexMove.BLOCK
    end,
    -- 判断是否正在受击
    IsGettingHit = function(self)
        return self.stun_state == StunType.BEING_PUNISHED or self.stun_state == StunType.GETTING_HIT
    end,
    IsPunish = function(self)
        -- 判断是否在惩罚状态
        return self.stun_state == StunType.BEING_PUNISHED
    end,
    -- 判断2帧内是否进行了动作变更
    IsMoveChanged = function(is_p1, self)
        state_log = tekken.get(is_p1, 2)
        return state_log.move_id ~= self.move_id
    end,
    -- 判断是否站立
    IsWhileStanding = function(self)
        return self.simple_state == SimpleMove.CROUCH or self.simple_state == SimpleMove.CROUCH_BACK or
            self.simple_state == SimpleMove.CROUCH_FORWARD
    end,

	GetHit = function(self)
		for i = 0, self.startup do  -- 回溯过去帧
			local stateLog = tekken.get(false, i)
			if stateLog and stateLog.move_timer == 1 then
				return i -- 找到 move_timer == 1 的快照，返回该快照
			end
		end
		return nil -- 如果没有找到，返回 nil
	end,
	
    -- 判断是否开始被扔
    IsStartedBeingThrown = function(self)
        stateLog = tekken.get(true, 2)
        return self.attack_type == AttackType.THROW and not stateLog.attack_type == AttackType.THROW
    end,
    -- 获取攻击的有效帧数
    GetActiveFrames = function(self)
        return self.startup_end - self.startup + 1
    end,
    getFa = function(attacker, receiver)
        -- 检查被攻击者的状态
        if receiver.simple_state == SimpleMove.KNOCKDOWN then
            return -2 -- 被击倒状态
        elseif receiver.hit_outcome == HitType.JUGGLE then
            return -1 -- 被击飞状态
        else
            local timeTillRecoveryP1 = attacker.frames_til_next_move
            local timeTillRecoveryP2 = receiver.hit_outcome == HitType.NONE and 0 or receiver.frames_til_next_move
            -- 计算原始帧优势
            local rawFa = timeTillRecoveryP2 - timeTillRecoveryP1
            return rawFa -- 返回帧优势
        end
    end,
    --检查是否挥空
    IsAttackWhiffing = function(self)
        return self.complex_state == ComplexMove.END1 or self.complex_state == ComplexMove.F_MINUS or
            self.complex_state == ComplexMove.RECOVERING or
            self.complex_state == ComplexMove.UN17 or
            self.complex_state == ComplexMove.UN17 or
            self.complex_state == ComplexMove.SS or
            self.complex_state == ComplexMove.WALK
    end,
    -- 检查挥空状态
    is_Whiffing = function(is_p1)
        local count = 0
        local damage = 0
        local count_size = tekken.count()
        if count_size > 1000 then
            count_size = 1000
        end
        local last_damage = tekken.get(not is_p1, 0).damage_taken
        for frame = 0, count_size do
            local p = tekken.get(not is_p1, frame)
            if p.move_id == 0 then
                break
            end
            if p.damage_taken ~= last_damage then
                count = count + 1
                damage = damage + (last_damage - p.damage_taken)
                last_damage = p.damage_taken
            end
            if
                p.hit_outcome == HitType.NONE or p.hit_outcome == HitType.BLOCKED_STANDING or
                    p.hit_outcome == HitType.BLOCKED_CROUCHING
             then
                if utils.IsAttackWhiffing(p) then
                    break
                end
            end
        end
        return count == 0
    end,
    -- 获取连招状态
    get_combo = function(is_p1)
        local count = 0
        local damage = 0
        local count_size = tekken.count()
        if count_size > 1000 then
            count_size = 1000
        end
        local last_damage = tekken.get(not is_p1, 0).damage_taken
        for frame = 0, count_size do
            local p = tekken.get(not is_p1, frame)
            if p.move_id == 0 then
                break
            end
            if p.damage_taken ~= last_damage then
                count = count + 1
                damage = damage + (last_damage - p.damage_taken)
                last_damage = p.damage_taken
            end
            if
                p.hit_outcome == HitType.NONE or p.hit_outcome == HitType.BLOCKED_STANDING or
                    p.hit_outcome == HitType.BLOCKED_CROUCHING
             then
                if
                    p.simple_state ~= SimpleMove.JUGGLED and p.simple_state ~= SimpleMove.WALL_SPLAT_18 and
                        p.simple_state ~= SimpleMove.WALL_SPLAT_19
                 then
                    break
                end
            end
        end
        return count, damage
    end,
    -- 检查是否开始攻击
    is_starting_attack = function(is_player1)
        local before = 2
        -- 获取当前帧之前的玩家快照
        local player = tekken.get(is_player1, before)
        -- 如果玩家正在进行攻击
        if player.move_timer ~= 0 then
            local cur_frame_count = tekken.GetFrameCount(1)
            local prev_frame_count = tekken.GetFrameCount(2)
            -- 计算丢帧数
            local dropped_frames = cur_frame_count - prev_frame_count - 1
            -- 计算攻击动作启动与攻击动作发生之间的时间差
            local diff = player.startup - player.move_timer - 1
            -- 如果时间差在丢帧数范围内
            if diff >= 0 and diff <= dropped_frames then
                -- 获取当前帧之前的第二个玩家快照
                local previous_player = tekken.get(is_player1, before + 1)
                -- 如果上一个玩家的移动定时器小于当前玩家的移动定时器
                if previous_player.move_timer < player.move_timer then
                    -- 攻击已经开始
                    return true
                end
            end
        end
        -- 攻击未开始或未完成
        return false
    end
}
