local M = {}

function M:OppIsThrowing(self, enemy)
    return utils.IsAttackThrow(enemy) or utils.IsStartedBeingThrown(enemy)
end

function M:getUp(self, enemy)
    if utils.IsOnGround(self) and self.input_direction == 32 and self.recovery == 1 then
        executeMacro(Input.BACK, 0, 3, InputType.KEY_PRESS)
    end
end

function M:AttackIfPossible(self, enemy)
	if config.USE_SCRIPT_COMBO_FUNCTION == false then
        return
	end
    local frames = utils.GetTimeUntilImpact(self)
    if selfScript ~= nil and selfScript.OnInitAttackProc then
        selfScript.OnInitAttackProc(self, enemy, frames - 1)
    end
    if enemy.stun_state == StunType.GETTING_HIT and enemy.complex_state == ComplexMove.RECOVERING then
        if enemy.hit_outcome >= HitType.NONE and self.distance <= 3000.0 then
            if utils.IsGettingCounterHit(enemy) then
                if selfScript ~= nil and selfScript.onAttackSuccess then
                    selfScript.onAttackCounter(self, enemy, frames)
                end
            else
                if selfScript ~= nil and selfScript.onAttackSuccess then
                    selfScript.onAttackSuccess(self, enemy, frames)
                end
            end
        end
    end
    return
end

function M:DefendAndCounter(self, enemy)
    if utils.IsAttackStarting(self) or utils.IsAttackThrow(enemy) then
        return
    end
    -- Get the number of frames until the opponent's attack impacts
    local frames = utils.GetTimeUntilImpact(enemy)

    -- Check if the enemy's attack has started
    if utils.IsAttackStarting(enemy) then
        -- If the opponent's attack frames have elapsed (no impact)
        math.randomseed(os.time())

        local autoBlockRate = math.random(0, 100)
        -- Proceed only if the opponent is within a reasonable distance and the attack is imminent
        if self.distance <= 2300.0 and frames > 0 then
            -- Handle high attacks with a chance for duck whiff punish or block
            if utils.IsAttackHigh(enemy) then
                -- Handle low attacks with the configured defense mechanism
                if config.DUCK_WHIP_PUNISH_CHANCE >= autoBlockRate and not utils.IsAirborne(enemy) and frames > 10 and frames < 15 then
					-- duck whiff punish if the attack is still in early stages
                    executeMacro(Input.BACK+Input.DOWN , 0, frames, InputType.KEY_DOWN)
					if selfScript and selfScript.duck_whiff_punish then
						selfScript.duck_whiff_punish(self, enemy, frames)
					end
                elseif config.BLOCK_HIGH_CHANCE >= autoBlockRate then
                    -- Block the high attack if whiff punish isn't triggered
                    executeMacro(Input.BACK, 0, 0, InputType.KEY_DOWN)
                end
            elseif utils.IsAttackLow(enemy) and config.BLOCK_LOW_CHANCE >= autoBlockRate then
                -- Handle mid attacks with the chance of a block
                local autoLowParryRate = math.random(0, 100)
                if config.LOW_PARRY_CHANCE >= autoLowParryRate then
                    executeMacro(Input.DOWN + Input.FORWARD, 0, frames, InputType.KEY_DOWN)
                else
                    executeMacro(Input.DOWN, 0, 0, InputType.KEY_DOWN)
                end
            elseif config.BLOCK_MID_CHANCE >= autoBlockRate and utils.IsAttackMid(enemy) then
                -- Block the mid attack
                executeMacro(Input.BACK, 0, 0, InputType.KEY_DOWN)
            end
        end
	end
    if frames <= 0 then
        -- Release any held keys to reset the input
        executeMacro(0, 0, 0, InputType.KEY_UP)
    end

    -- Check for Block Counter
	local Index = utils.GetHit(enemy)
    if Index and self.distance <= 2000.0 then
        local isOpponentAirborne = utils.IsAirborne(enemy)
        local isOpponentLowAttack = utils.IsAttackLow(enemy)
        local isOpponentMidAttack = utils.IsAttackMid(enemy)
        local isOpponentHighAttack = not isOpponentLowAttack and not isOpponentMidAttack
		local frameDifference = enemy.frames_til_next_move - self.frames_til_next_move
		-- Access player and enemy states
		local self = tekken.get(true, Index)
		local enemy = tekken.get(false, Index)

        math.randomseed(os.time())
        local autoBlockPunishChance = math.random(0, 100)

        if frameDifference > 0 and frameDifference < 25 and config.BLOCK_PUNISH_CHANCE >= autoBlockPunishChance then
            if isOpponentAirborne and selfScript and selfScript.air_counters then
                selfScript.air_counters(self, enemy, frameDifference)
                return 1
            elseif isOpponentLowAttack and selfScript and selfScript.low_counters then
                selfScript.low_counters(self, enemy, frameDifference)
                return 1
            elseif isOpponentMidAttack and selfScript and selfScript.mid_counters then
                selfScript.mid_counters(self, enemy, frameDifference)
                return 1
            elseif isOpponentHighAttack and selfScript and selfScript.high_counters then
                selfScript.high_counters(self, enemy, frameDifference)
                return 1
            end
        end
    end
end

return M
